function [rho,gamma] = autocorr(x,k) ;
% Computes the first k autocovariance and autocorrelation coefficients of x
% 	Usage: [rho,gamma] = autocorr(x,k) ;
%
%		INPUT (required)
%	x = time series
%
%		INPUT (optional)
%	k = number of lags at which the autocorr is calculated
%		(default = min([n-1,20]))
n=length(x) ;

if nargin < 2 ;
	k = min([n-1,20]) ;
end ;

xc = x - mean(x) ;

gamma = zeros(k+1,1) ;

for i = 0:k ;
gamma(i+1) = sum( xc(1:n-i) .* xc(i+1:n) ) ;
end ;

gamma = gamma/n ;
rho = gamma(2:k+1) / gamma(1) ;

