function B=bspline(x,ndx,bdeg,xl,xr) ;
%	
%	B-spline basis functions using a program of Eilers and Marx
%	Requires equally spaced-knots, which are generated by the program
%	USAGE: B=bspline(x,ndx,bdeg,xl,xr) ;
%
%		OUTPUT
%	B = basis functions evaluated at x
%
%		INPUT (Required)
%	x = points where the B-splines will be evaluated
%
%		INPUT (Optional)
%	ndx = number of intervals in the domain (default = 20)
%	bdeg = degree of the B-splines (default = 2)
%	xl = left limit of x-intervals (default = min(x) - 100*eps)
%	xr = right limit x-intervals (default = max(x) + 100*eps)
%
if (nargin < 5 | xr = []) ;
	xr = max(x) + 1000*eps;
end ;

if (nargin < 4 | xl = []) ;
	xl = min(x) - 1000*eps ;
end ;

if (nargin < 3 | bdeg = []) ;
	bdeg = 2 ;
end ;

if( nargin < 2 | ndx = []) ;
	ndx = 20 ;
end ;

dx = (xr-xl)/ndx;
t=xl+dx*[-bdeg:ndx-1] ;
T=(0*x+1)*t;
X=x*(0*t +1);
P=(X-T)/dx ;
B=(T<=X) & (X < (T+dx)) ;
r=[2:length(t) 1]; 
for k=1:bdeg
	B=(P.*B + (k+1-P).*B(:,r)) / k ;
end ;
