function [betahat,mhat,sigma2,varbetahat] = fastspam(x,z,y,b,p) ;
%
%  	Syntax:  [betahat,mhat,sigma2,varbetahat] = fastspam(x,z,y,b,p) 
%
%	x = variables used parametrically
%	z = variables used nonparametrically
%	y = dependent variable
%	b = bandwidth vector
%	p = degree of local polynomials
%
%	betahat = estimated coefficients of x in model
%	mhat = estimated additive functions for z
%	sigma2 = estimated residual variance
%	varbetahat = estimated variance-covariance matrix of betahat
%
%		Last edited:	December 10, 1996
%
%		Changed from 12/9/96 version --- now adds column of
% 		ones when fitting the x variables (so there's no need
%		to worry about centering or not centering the x's
%		and y's)
%
M1 = 30 ;
n = rows(x) ;
mx = cols(x) ;
mz = cols(z) ;
mx1 = mx+1 ;                     		%  Number of x variables with
						%  intercept used
xd = [z x ones(n,1)] ;
betazx = inv(xd'*xd)*xd'*y ;
mstart = z.*(ones(n,1)*betazx(1:mz)') ;  %  Starting values use a linear
					 % fit to both the x's and z's


xfit = zeros(n,mx) ;


	for i = 1:mx ;
	[fit,smat] = backfit(x(:,i),z,b,p,mstart,M1) ;
	xfit(:,i) = fit*ones(cols(z),1) ;	%  Sum over fitted
						% functions of the z's
	end ;

[yfit,smat] = backfit(y,z,b,p,mstart,M1) ;
yfit = yfit*ones(cols(z),1) ;
c=x-xfit ;
c=[ones(n,1) c] ;
x2 = [ones(n,1) x] ;                                 % Add column of ones
B = x2'*c ;

invb = inv(B) ;
a = c'*c ;
betahat = invb*x2'*(y-yfit) ;
betahat=betahat(2:mx1) ;                           % Remove intercept

xbeta = x*betahat ;

[mhat,smat] = backfit(y-xbeta,z,b,p,mstart,M1) ;

% The stuff below needs to be developed (df could be refined)
%

e = y - xbeta - mhat*ones(cols(z),1) ;
df= 0 ;
sigma2 = norm(e)^2 / (n-df) ;
varbetahat = sigma2 * invb*a*invb ;
varbetahat = varbetahat(2:mx1,2:mx1) ;            %  Remove intercept
