function [yhat,hmin,gcv,span] = gcv(x,y,k,span) ;
% Program to compute a local polynomial estimate with a gcv selected span
%
%		INPUT
%	x = independent variable (n by 1)
%	y = dependent variable (n by 1)
%	span = vector of possible values of the span (this program
%		finds the value that minimizes cv (default is
%		linspace((4+2*k)/n,1,20))
%	k = degree of the local polynomials (default is 1)
%
%		OUTPUT
%	yhat = estimate at span that minimizes cv (n by 1)
%	hmin = vector of bandwidth corresponding to cv chosen span
%		(n by 1)
%	cv = value of cv as a function of span (dimension = length(span) by 1)
%
% 	USAGE: [yhat,hmin,cv] = cvband(x,y,span,k) ;
%	CALLS: lpolycv
%
%	Last edit: 7/24/97
%	Copyright: David Ruppert
%
n = length(x) ;
if nargin < 4 ;
span = linspace((4+2*k)/n,1,20) ;
end ;
if nargin < 3 ;
k = 1 ;
end ;

h = zeros(length(x),length(span)) ;
gcv = zeros(length(span),1) ;

for i=1:length(span) ;
h(:,i) = bspan(x,x,span(i)) ;
[f(:,i),var,hat,gcv(i)]=lpolydb(x,y,h(:,i),x,0,k) ;
end ;

[gcvmin,imin]=min(gcv) ;
hmin = h(:,imin) ;
yhat= lpolydb(x,y,hmin) ;
